@@ -11,9 +11,9 @@ read from an internet service (e.g. Yahoo weather) and exposed to
 ths KNX bus. KNX sensors may show this outside temperature within their
 LCD display.
 """
+from xknx.remote_value import RemoteValueSensor, RemoteValueSwitch
+
 from .device import Device
-from .remote_value_scaling import RemoteValueScaling
-from .remote_value_sensor import RemoteValueSensor
 
 
 class ExposeSensor(Device):
@@ -27,23 +27,29 @@ class ExposeSensor(Device):
                  device_updated_cb=None):
         """Initialize Sensor class."""
         # pylint: disable=too-many-arguments
-        super(ExposeSensor, self).__init__(xknx, name, device_updated_cb)
+        super().__init__(xknx, name, device_updated_cb)
 
         self.sensor_value = None
-        if value_type == "percent":
-            self.sensor_value = RemoteValueScaling(
+        if value_type == "binary":
+            self.sensor_value = RemoteValueSwitch(
                 xknx,
                 group_address=group_address,
+                sync_state=False,
                 device_name=self.name,
                 after_update_cb=self.after_update)
         else:
             self.sensor_value = RemoteValueSensor(
                 xknx,
                 group_address=group_address,
+                sync_state=False,
                 device_name=self.name,
                 after_update_cb=self.after_update,
                 value_type=value_type)
 
+    def _iter_remote_values(self):
+        """Iterate the devices RemoteValue classes."""
+        yield self.sensor_value
+
     @classmethod
     def from_config(cls, xknx, name, config):
         """Initialize object from configuration structure."""
@@ -57,14 +63,6 @@ class ExposeSensor(Device):
                    group_address=group_address,
                    value_type=value_type)
 
-    def has_group_address(self, group_address):
-        """Test if device has given group address."""
-        return self.sensor_value.has_group_address(group_address)
-
-    def state_addresses(self):
-        """Return group addresses which should be requested to sync state."""
-        return []
-
     async def process_group_read(self, telegram):
         """Process incoming GROUP READ telegram."""
         await self.sensor_value.send(response=True)
@@ -89,7 +87,3 @@ class ExposeSensor(Device):
                     self.sensor_value.group_addr_str(),
                     self.resolve_state(),
                     self.unit_of_measurement())
-
-    def __eq__(self, other):
-        """Equal operator."""
-        return self.__dict__ == other.__dict__
