@@ -6,8 +6,9 @@ It provides functionality for
 * setting fan to specific speed
 * reading the current speed from KNX bus.
 """
+from xknx.remote_value import RemoteValueScaling
+
 from .device import Device
-from .remote_value_scaling import RemoteValueScaling
 
 
 class Fan(Device):
@@ -31,10 +32,15 @@ class Fan(Device):
             group_address_speed,
             group_address_speed_state,
             device_name=self.name,
+            feature_name="Speed",
             after_update_cb=self.after_update,
             range_from=0,
             range_to=100)
 
+    def _iter_remote_values(self):
+        """Iterate the devices RemoteValue classes."""
+        yield self.speed
+
     @classmethod
     def from_config(cls, xknx, name, config):
         """Initialize object from configuration structure."""
@@ -49,10 +55,6 @@ class Fan(Device):
             group_address_speed=group_address_speed,
             group_address_speed_state=group_address_speed_state)
 
-    def has_group_address(self, group_address):
-        """Test if device has given group address."""
-        return self.speed.has_group_address(group_address)
-
     def __str__(self):
         """Return object as readable string."""
         return '<Fan name="{0}" ' \
@@ -72,12 +74,6 @@ class Fan(Device):
         else:
             self.xknx.logger.warning("Could not understand action %s for device %s", action, self.get_name())
 
-    def state_addresses(self):
-        """Return group addresses which should be requested to sync state."""
-        state_addresses = []
-        state_addresses.extend(self.speed.state_addresses())
-        return state_addresses
-
     async def process_group_write(self, telegram):
         """Process incoming GROUP WRITE telegram."""
         await self.speed.process(telegram)
@@ -86,7 +82,3 @@ class Fan(Device):
     def current_speed(self):
         """Return current speed of fan."""
         return self.speed.value
-
-    def __eq__(self, other):
-        """Equal operator."""
-        return self.__dict__ == other.__dict__
