@@ -6,11 +6,15 @@ class XKNXException(Exception):
 
     def __eq__(self, other):
         """Equal operator."""
-        return self.__dict__ == other.__dict__
+        return repr(self) == repr(other)
 
     def __hash__(self):
         """Hash function."""
-        return 0
+        return hash(str(self))
+
+    def __repr__(self):
+        """Representation of object."""
+        return str(self)
 
 
 class CouldNotParseTelegram(XKNXException):
@@ -18,7 +22,7 @@ class CouldNotParseTelegram(XKNXException):
 
     def __init__(self, description, **kwargs):
         """Initialize CouldNotParseTelegram class."""
-        super(CouldNotParseTelegram, self).__init__("Could not parse Telegram")
+        super().__init__()
         self.description = description
         self.parameter = kwargs
 
@@ -32,11 +36,11 @@ class CouldNotParseTelegram(XKNXException):
 
 
 class CouldNotParseKNXIP(XKNXException):
-    """Exception falss for worong KNXIP data."""
+    """Exception class for wrong KNXIP data."""
 
     def __init__(self, description=""):
-        """Initialize CouldNotParseTelegram class."""
-        super(CouldNotParseKNXIP, self).__init__("Could not parse KNXIP")
+        """Initialize CouldNotParseKNXIP class."""
+        super().__init__()
         self.description = description
 
     def __str__(self):
@@ -45,12 +49,26 @@ class CouldNotParseKNXIP(XKNXException):
             .format(self.description)
 
 
+class UnsupportedCEMIMessage(XKNXException):
+    """Exception class for unsupported CEMI Messages."""
+
+    def __init__(self, description=""):
+        """Initialize UnsupportedCEMIMessage class."""
+        super().__init__()
+        self.description = description
+
+    def __str__(self):
+        """Return object as readable string."""
+        return '<UnsupportedCEMIMessage description="{0}" />' \
+            .format(self.description)
+
+
 class ConversionError(XKNXException):
     """Exception class for error while converting one type to another."""
 
     def __init__(self, description, **kwargs):
         """Initialize ConversionError class."""
-        super(ConversionError, self).__init__("Conversion Error")
+        super().__init__()
         self.description = description
         self.parameter = kwargs
 
@@ -67,7 +85,7 @@ class CouldNotParseAddress(XKNXException):
 
     def __init__(self, address=None):
         """Initialize CouldNotParseAddress class."""
-        super(CouldNotParseAddress, self).__init__("Could not parse address")
+        super().__init__()
         self.address = address
 
     def __str__(self):
@@ -80,7 +98,7 @@ class DeviceIllegalValue(XKNXException):
 
     def __init__(self, value, description):
         """Initialize DeviceIllegalValue class."""
-        super(DeviceIllegalValue, self).__init__("Illegal value for device")
+        super().__init__()
         self.value = value
         self.description = description
 
