@@ -6,8 +6,9 @@ Tunnels connect to KNX/IP devices directly via UDP and build a static UDP connec
 import asyncio
 
 from xknx.exceptions import XKNXException
-from xknx.knx import TelegramDirection
-from xknx.knxip import KNXIPFrame, KNXIPServiceType, TunnellingRequest
+from xknx.knxip import (
+    CEMIMessageCode, KNXIPFrame, KNXIPServiceType, TunnellingRequest)
+from xknx.telegram import TelegramDirection
 
 from .connect import Connect
 from .connectionstate import ConnectionState
@@ -63,13 +64,17 @@ class Tunnel():
             self.xknx.logger.warning("Service not implemented: %s", knxipframe)
         else:
             self.send_ack(knxipframe.body.communication_channel_id, knxipframe.body.sequence_counter)
-            telegram = knxipframe.body.cemi.telegram
-            telegram.direction = TelegramDirection.INCOMING
-            if self.telegram_received_callback is not None:
-                self.telegram_received_callback(telegram)
+            # Don't handle invalid cemi frames (None) and only handle incoming L_DATA_IND frames.
+            # Ignore L_DATA_CON confirmation frames. L_DATA_REQ frames should only be outgoing.
+            if knxipframe.body.cemi is not None and \
+                    knxipframe.body.cemi.code is CEMIMessageCode.L_DATA_IND:
+                telegram = knxipframe.body.cemi.telegram
+                telegram.direction = TelegramDirection.INCOMING
+                if self.telegram_received_callback is not None:
+                    self.telegram_received_callback(telegram)
 
     def send_ack(self, communication_channel_id, sequence_counter):
-        """Send tunneling ACK after tunneling request received."""
+        """Send tunnelling ACK after tunnelling request received."""
         ack_knxipframe = KNXIPFrame(self.xknx)
         ack_knxipframe.init(KNXIPServiceType.TUNNELLING_ACK)
         ack_knxipframe.body.communication_channel_id = communication_channel_id
@@ -113,7 +118,7 @@ class Tunnel():
 
     async def send_telegram(self, telegram):
         """
-        Send Telegram to routing tunelling device - retry mechanism.
+        Send Telegram to routing tunnelling device - retry mechanism.
 
         If a TUNNELLING_REQUEST frame is not confirmed within the TUNNELLING_REQUEST_TIME_- OUT
         time of one (1) second then the frame shall be repeated once with the same sequence counter
@@ -139,7 +144,7 @@ class Tunnel():
         self.increase_sequence_number()
 
     async def _send_telegram_impl(self, telegram):
-        """Send Telegram to routing tunelling device - implementation."""
+        """Send Telegram to tunnelling device - implementation."""
         tunnelling = Tunnelling(
             self.xknx,
             self.udp_client,
@@ -179,8 +184,7 @@ class Tunnel():
         await disconnect.start()
         if not disconnect.success and not ignore_error:
             raise XKNXException("Could not disconnect channel")
-        else:
-            self.xknx.logger.debug("Tunnel disconnected (communication_channel: %s)", self.communication_channel)
+        self.xknx.logger.debug("Tunnel disconnected (communication_channel: %s)", self.communication_channel)
         # close udp client to prevent open file descriptors
         await self.udp_client.stop()
 
