@@ -11,26 +11,26 @@ Documentation within:
     KNX IP Communication Medium
     File: AN117 v02.01 KNX IP Communication Medium DV.pdf
 """
-from xknx.exceptions import ConversionError, CouldNotParseKNXIP
-from xknx.knx import (
-    DPTArray, DPTBinary, GroupAddress, PhysicalAddress, Telegram, TelegramType)
+from xknx.dpt import DPTArray, DPTBinary
+from xknx.exceptions import (
+    ConversionError, CouldNotParseKNXIP, UnsupportedCEMIMessage)
+from xknx.telegram import GroupAddress, PhysicalAddress, Telegram, TelegramType
 
-from .body import KNXIPBody
 from .knxip_enum import APCICommand, CEMIFlags, CEMIMessageCode
 
 
-class CEMIFrame(KNXIPBody):
+class CEMIFrame():
     """Representation of a CEMI Frame."""
 
     # pylint: disable=too-many-instance-attributes
 
     def __init__(self, xknx):
         """Initialize CEMIFrame object."""
-        super(CEMIFrame, self).__init__(xknx)
+        self.xknx = xknx
         self.code = CEMIMessageCode.L_DATA_IND
         self.flags = 0
         self.cmd = APCICommand.GROUP_READ
-        self.src_addr = GroupAddress(None)
+        self.src_addr = PhysicalAddress(None)
         self.dst_addr = GroupAddress(None)
         self.mpdu_len = 0
         self.payload = None
@@ -105,24 +105,29 @@ class CEMIFrame(KNXIPBody):
 
     def from_knx(self, raw):
         """Parse/deserialize from KNX/IP raw data."""
-        self.code = CEMIMessageCode(raw[0])
+        try:
+            self.code = CEMIMessageCode(raw[0])
+        except ValueError:
+            raise UnsupportedCEMIMessage("CEMIMessageCode not implemented: {0} ".format(raw[0]))
 
-        if self.code == CEMIMessageCode.L_DATA_IND or \
-                self.code == CEMIMessageCode.L_Data_REQ or \
-                self.code == CEMIMessageCode.L_DATA_CON:
-            return self.from_knx_data_link_layer(raw)
-        raise CouldNotParseKNXIP("Could not understand CEMIMessageCode: {0} / {1}".format(self.code, raw[0]))
+        if self.code not in (CEMIMessageCode.L_DATA_IND,
+                             CEMIMessageCode.L_Data_REQ,
+                             CEMIMessageCode.L_DATA_CON):
+            raise UnsupportedCEMIMessage("Could not handle CEMIMessageCode: {0} / {1}".format(self.code, raw[0]))
+
+        return self.from_knx_data_link_layer(raw)
 
     def from_knx_data_link_layer(self, cemi):
         """Parse L_DATA_IND, CEMIMessageCode.L_Data_REQ, CEMIMessageCode.L_DATA_CON."""
         if len(cemi) < 11:
-            raise CouldNotParseKNXIP("CEMI too small")
+            # eg. ETS Line-Scan issues L_DATA_IND with length 10
+            raise UnsupportedCEMIMessage("CEMI too small. Length: {0}; CEMI: {1}".format(len(cemi), cemi))
 
         # AddIL (Additional Info Length), as specified within
         # KNX Chapter 3.6.3/4.1.4.3 "Additional information."
         # Additional information is not yet parsed.
         addil = cemi[1]
-
+        # Control field 1 and Control field 2 - first 2 octets after Additional information
         self.flags = cemi[2 + addil] * 256 + cemi[3 + addil]
 
         self.src_addr = PhysicalAddress((cemi[4 + addil], cemi[5 + addil]))
@@ -140,7 +145,11 @@ class CEMIFrame(KNXIPBody):
 
         tpci_apci = cemi[9 + addil] * 256 + cemi[10 + addil]
 
-        self.cmd = APCICommand(tpci_apci & 0xFFC0)
+        try:
+            self.cmd = APCICommand(tpci_apci & 0xFFC0)
+        except ValueError:
+            raise UnsupportedCEMIMessage(
+                "APCI not supported: {0:#012b}".format(tpci_apci & 0xFFC0))
 
         apdu = cemi[10 + addil:]
         if len(apdu) != self.mpdu_len:
