@@ -17,8 +17,8 @@ class SearchRequest(KNXIPBody):
 
     def __init__(self, xknx):
         """Initialize SearchRequest object."""
-        super(SearchRequest, self).__init__(xknx)
-        self.discovery_endpoint = HPAI(ip_addr="224.0.23.12", port=3671)
+        super().__init__(xknx)
+        self.discovery_endpoint = HPAI(ip_addr=xknx.multicast_group, port=xknx.multicast_port)
 
     def calculated_length(self):
         """Get length of KNX/IP body."""
