@@ -4,10 +4,13 @@ import logging
 import signal
 from sys import platform
 
-from xknx.core import Config, TelegramQueue
+from xknx.core import Config, StateUpdater, TelegramQueue
 from xknx.devices import Devices
-from xknx.io import ConnectionConfig, KNXIPInterface
-from xknx.knx import GroupAddressType, PhysicalAddress
+from xknx.io import (
+    DEFAULT_MCAST_GRP, DEFAULT_MCAST_PORT, ConnectionConfig, KNXIPInterface)
+from xknx.telegram import GroupAddressType, PhysicalAddress
+
+from .__version__ import __version__ as VERSION
 
 
 class XKNX:
@@ -16,6 +19,7 @@ class XKNX:
     # pylint: disable=too-many-instance-attributes
 
     DEFAULT_ADDRESS = '15.15.250'
+    DEFAULT_RATE_LIMIT = 20
 
     def __init__(self,
                  config=None,
@@ -23,7 +27,10 @@ class XKNX:
                  own_address=PhysicalAddress(DEFAULT_ADDRESS),
                  address_format=GroupAddressType.LONG,
                  telegram_received_cb=None,
-                 device_updated_cb=None):
+                 device_updated_cb=None,
+                 rate_limit=DEFAULT_RATE_LIMIT,
+                 multicast_group=DEFAULT_MCAST_GRP,
+                 multicast_port=DEFAULT_MCAST_PORT):
         """Initialize XKNX class."""
         # pylint: disable=too-many-arguments
         self.devices = Devices()
@@ -31,14 +38,20 @@ class XKNX:
         self.loop = loop or asyncio.get_event_loop()
         self.sigint_received = asyncio.Event()
         self.telegram_queue = TelegramQueue(self)
-        self.state_updater = None
+        self.state_updater = StateUpdater(self)
         self.knxip_interface = None
-        self.started = False
+        self.started = asyncio.Event()
         self.address_format = address_format
         self.own_address = own_address
+        self.rate_limit = rate_limit
+        self.multicast_group = multicast_group
+        self.multicast_port = multicast_port
         self.logger = logging.getLogger('xknx.log')
         self.knx_logger = logging.getLogger('xknx.knx')
         self.telegram_logger = logging.getLogger('xknx.telegram')
+        self.raw_socket_logger = logging.getLogger('xknx.raw_socket')
+        self.connection_config = None
+        self.version = VERSION
 
         if config is not None:
             Config(self).read(config)
@@ -51,7 +64,7 @@ class XKNX:
 
     def __del__(self):
         """Destructor. Cleaning up if this was not done before."""
-        if self.started:
+        if self.started.is_set():
             try:
                 task = self.loop.create_task(self.stop())
                 self.loop.run_until_complete(task)
@@ -61,22 +74,24 @@ class XKNX:
     async def start(self,
                     state_updater=False,
                     daemon_mode=False,
-                    connection_config=ConnectionConfig()):
+                    connection_config=None):
         """Start XKNX module. Connect to KNX/IP devices and start state updater."""
+        if connection_config is None:
+            if self.connection_config is None:
+                connection_config = ConnectionConfig()
+            else:
+                connection_config = self.connection_config
         self.knxip_interface = KNXIPInterface(self, connection_config=connection_config)
+        self.logger.info('XKNX v%s starting %s connection to KNX bus.',
+                         VERSION, connection_config.connection_type.name.lower())
         await self.knxip_interface.start()
         await self.telegram_queue.start()
-
         if state_updater:
-            from xknx.core import StateUpdater
-            self.state_updater = StateUpdater(self)
-            await self.state_updater.start()
-
+            self.state_updater.start()
+        self.started.set()
         if daemon_mode:
             await self.loop_until_sigint()
 
-        self.started = True
-
     async def join(self):
         """Wait until all telegrams were processed."""
         await self.telegrams.join()
@@ -89,10 +104,11 @@ class XKNX:
 
     async def stop(self):
         """Stop XKNX module."""
+        self.state_updater.stop()
         await self.join()
         await self.telegram_queue.stop()
         await self._stop_knxip_interface_if_exists()
-        self.started = False
+        self.started.clear()
 
     async def loop_until_sigint(self):
         """Loop until Crtl-C was pressed."""
